import re
import datetime
from dataclasses import dataclass
from typing import Type, Union, Iterable, Optional, TypedDict, overload

from nonebot.typing import overrides
from nonebot.utils import escape_tag

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .utils import escape
from .api import (
    File,
    Embed,
    Button,
    ActionRow,
    Component,
    Snowflake,
    MessageGet,
    SelectMenu,
    SnowflakeType,
    AttachmentSend,
    TimeStampStyle,
    DirectComponent,
    MessageReference,
)


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @overrides(BaseMessageSegment)
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @staticmethod
    def attachment(
        file: Union[str, File, AttachmentSend],
        description: Optional[str] = None,
        content: Optional[bytes] = None,
    ) -> "AttachmentSegment":
        if isinstance(file, str):
            _filename = file
            _description = description
            _content = content
        elif isinstance(file, File):
            _filename = file.filename
            _description = description
            _content = file.content
        elif isinstance(file, AttachmentSend):
            _filename = file.filename
            _description = file.description
            _content = content
        else:
            raise TypeError("file must be str, File or AttachmentSend")
        if _content is None:
            return AttachmentSegment(
                data={
                    "attachment": AttachmentSend(
                        filename=_filename, description=_description
                    ),
                    "file": None,
                }
            )
        else:
            return AttachmentSegment(
                data={
                    "attachment": AttachmentSend(
                        filename=_filename, description=_description
                    ),
                    "file": File(filename=_filename, content=_content),
                }
            )

    @staticmethod
    def sticker(sticker_id: SnowflakeType) -> "StickerSegment":
        return StickerSegment(data={"id": Snowflake(sticker_id)})

    @staticmethod
    def embed(embed: Embed) -> "EmbedSegment":
        return EmbedSegment(data={"embed": embed})

    @staticmethod
    def component(component: Component):
        if isinstance(component, (Button, SelectMenu)):
            component = ActionRow(components=[component])
        return ComponentSegment(data={"component": component})

    @staticmethod
    def custom_emoji(
        name: str, emoji_id: str, animated: Optional[bool] = None
    ) -> "CustomEmojiSegment":
        return CustomEmojiSegment(
            data={"name": name, "id": emoji_id, "animated": animated}
        )

    @staticmethod
    def mention_user(user_id: SnowflakeType) -> "MentionUserSegment":
        return MentionUserSegment(data={"user_id": Snowflake(user_id)})

    @staticmethod
    def mention_role(role_id: SnowflakeType) -> "MentionRoleSegment":
        return MentionRoleSegment(data={"role_id": Snowflake(role_id)})

    @staticmethod
    def mention_channel(channel_id: SnowflakeType) -> "MentionChannelSegment":
        return MentionChannelSegment(data={"channel_id": Snowflake(channel_id)})

    @staticmethod
    def mention_everyone() -> "MentionEveryoneSegment":
        return MentionEveryoneSegment()

    @staticmethod
    def text(content: str) -> "TextSegment":
        return TextSegment(data={"text": escape(content)})

    @staticmethod
    def timestamp(
        timestamp: Union[int, datetime.datetime], style: Optional[TimeStampStyle] = None
    ) -> "TimestampSegment":
        if isinstance(timestamp, datetime.datetime):
            timestamp = int(timestamp.timestamp())
        return TimestampSegment(data={"timestamp": timestamp, "style": style})

    @staticmethod
    @overload
    def reference(reference: MessageReference) -> "ReferenceSegment":
        ...

    @staticmethod
    @overload
    def reference(
        reference: SnowflakeType,
        channel_id: Optional[SnowflakeType] = None,
        guild_id: Optional[SnowflakeType] = None,
        fail_if_not_exists: Optional[bool] = None,
    ) -> "ReferenceSegment":
        ...

    @staticmethod
    def reference(
        reference: Union[SnowflakeType, MessageReference],
        channel_id: Optional[SnowflakeType] = None,
        guild_id: Optional[SnowflakeType] = None,
        fail_if_not_exists: Optional[bool] = None,
    ):
        if isinstance(reference, MessageReference):
            _reference = reference
        else:
            _reference = MessageReference.parse_obj(
                {
                    "message_id": reference,
                    "channel_id": channel_id,
                    "guild_id": guild_id,
                    "fail_if_not_exists": fail_if_not_exists,
                }
            )

        return ReferenceSegment(data={"reference": _reference})

    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        return self.type == "text"


@dataclass
class StickerSegment(MessageSegment):
    type: str = "sticker"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Sticker:{self.data['id']}>"


ComponentData = TypedDict("ComponentData", {"component": DirectComponent})


@dataclass
class ComponentSegment(MessageSegment):
    type: str = "component"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Component:{self.data['component'].type}>"


CustomEmojiData = TypedDict(
    "CustomEmojiData", {"name": str, "id": str, "animated": Optional[bool]}
)


@dataclass
class CustomEmojiSegment(MessageSegment):
    type: str = "custom_emoji"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        if self.data.get("animated"):
            return f"<a:{self.data['name']}:{self.data['id']}>"
        else:
            return f"<:{self.data['name']}:{self.data['id']}>"


MentionUserData = TypedDict("MentionUserData", {"user_id": Snowflake})


@dataclass
class MentionUserSegment(MessageSegment):
    type: str = "mention_user"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<@{self.data['user_id']}>"


MentionChannelData = TypedDict("MentionChannelData", {"channel_id": Snowflake})


@dataclass
class MentionChannelSegment(MessageSegment):
    type: str = "mention_channel"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<#{self.data['channel_id']}>"


MentionRoleData = TypedDict("MentionRoleData", {"role_id": Snowflake})


@dataclass
class MentionRoleSegment(MessageSegment):
    type: str = "mention_role"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<@&{self.data['role_id']}>"


@dataclass
class MentionEveryoneSegment(MessageSegment):
    type: str = "mention_everyone"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return "@everyone"


TimestampData = TypedDict(
    "TimestampData", {"timestamp": int, "style": Optional[TimeStampStyle]}
)


@dataclass
class TimestampSegment(MessageSegment):
    type: str = "timestamp"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        style = self.data.get("style")
        return (
            f"<t:{self.data['timestamp']}"
            + (
                f":{style.value if isinstance(style, TimeStampStyle) else style}"
                if style
                else ""
            )
            + ">"
        )


TextData = TypedDict("TextData", {"text": str})


@dataclass
class TextSegment(MessageSegment):
    type: str = "text"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return escape_tag(self.data["text"])


EmbedData = TypedDict("EmbedData", {"embed": Embed})


@dataclass
class EmbedSegment(MessageSegment):
    type: str = "embed"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Embed:{self.data['embed'].type}>"


AttachmentData = TypedDict(
    "AttachmentData", {"attachment": AttachmentSend, "file": Optional[File]}
)


@dataclass
class AttachmentSegment(MessageSegment):
    type: str = "attachment"

    @overrides(MessageSegment)
    def __str__(self) -> str:
        return f"<Attachment:{self.data['attachment'].filename}>"


ReferenceData = TypedDict("ReferenceData", {"reference": MessageReference})


@dataclass
class ReferenceSegment(MessageSegment):
    type: str = "reference"

    @overrides(MessageSegment)
    def __str__(self):
        return f"<Reference:{self.data['reference'].message_id}>"


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @overrides(BaseMessage)
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @overrides(BaseMessage)
    def __add__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super(Message, self).__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @overrides(BaseMessage)
    def __radd__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super(Message, self).__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @staticmethod
    @overrides(BaseMessage)
    def _construct(msg: str) -> Iterable[MessageSegment]:
        text_begin = 0
        for embed in re.finditer(
            r"<(?P<type>(@!|@&|@|#|/|:|a:|t:))?(?P<param>.+?)>",
            msg,
        ):
            if content := msg[text_begin : embed.pos + embed.start()]:
                yield TextSegment(data={"text": escape(content)})
            text_begin = embed.pos + embed.end()
            if embed.group("type") in ("@!", "@"):
                yield MentionUserSegment(
                    data={"user_id": Snowflake(embed.group("param"))}
                )
            elif embed.group("type") == "@&":
                yield MentionRoleSegment(
                    data={"role_id": Snowflake(embed.group("param"))}
                )
            elif embed.group("type") == "#":
                yield MentionChannelSegment(
                    data={"channel_id": Snowflake(embed.group("param"))}
                )
            elif embed.group("type") == "/":
                # TODO: slash command
                pass
            elif embed.group("type") in (":", "a:"):
                if len(cut := embed.group("param").split(":")) == 2:
                    yield CustomEmojiSegment(
                        data={
                            "name": cut[0],
                            "id": cut[1],
                            "animated": embed.group("type") == "a:",
                        }
                    )
                else:
                    yield TextSegment(data={"text": escape(embed.group())})
            else:
                if (
                    len(cut := embed.group("param").split(":")) == 2
                    and cut[0].isdigit()
                ):
                    yield TimestampSegment(
                        data={"timestamp": int(cut[0]), "style": TimeStampStyle(cut[1])}
                    )
                elif embed.group().isdigit():
                    yield TimestampSegment(
                        data={"timestamp": int(embed.group()), "style": None}
                    )
                else:
                    yield TextSegment(data={"text": escape(embed.group())})
        if content := msg[text_begin:]:
            yield TextSegment(data={"text": escape(content)})

    @classmethod
    def from_guild_message(cls, message: MessageGet) -> "Message":
        msg = Message()
        if message.mention_everyone:
            msg.append(MessageSegment.mention_everyone())
        if message.content:
            msg.extend(Message(message.content))
        if message.attachments:
            msg.extend(
                AttachmentSegment(
                    data={
                        "attachment": AttachmentSend(
                            filename=attachment.filename,
                            description=attachment.description or None,
                        ),
                        "file": None,
                    }
                )
                for attachment in message.attachments
            )
        if message.embeds:
            msg.extend(EmbedSegment(data={"embed": embed}) for embed in message.embeds)
        if message.components:
            msg.extend(
                ComponentSegment(data={"component": component})
                for component in message.components
            )
        return msg

    def extract_content(self) -> str:
        return "".join(
            str(seg)
            for seg in self
            if seg.type
            in (
                "text",
                "custom_emoji",
                "mention_user",
                "mention_role",
                "mention_everyone",
                "mention_channel",
                "timestamp",
            )
        )



from pathlib import Path

import aiofiles
import markdown
from jinja2 import Environment, FileSystemLoader
from nonebot import require

require("nonebot_plugin_htmlrender")
from nonebot_plugin_htmlrender import html_to_pic

require("nonebot_plugin_templates")
from nonebot_plugin_templates.templates_render import Font_Path

TEMPLATES_PATH = Path(__file__).parent / "templates"
env = Environment(
    extensions=["jinja2.ext.loopcontrols"],
    loader=FileSystemLoader(str(TEMPLATES_PATH)),
    enable_async=True,
)
text_template = env.get_template("markdown.html")


async def read_file(path: str) -> str:
    async with aiofiles.open(path, mode="r") as f:
        return await f.read()


async def read_tpl(path: str) -> str:
    return await read_file(f"{TEMPLATES_PATH}/{path}")


async def md_to_pic(md: str, width: int = 600, font_path: str = Font_Path):
    md = md.replace("\n", "  \n")
    if md.count("```") % 2 == 1:
        md += "  \n```"
    md = markdown.markdown(
        md,
        extensions=[
            "pymdownx.tasklist",
            "tables",
            "fenced_code",
            "codehilite",
            "mdx_math",
            "pymdownx.tilde",
        ],
        extension_configs={"mdx_math": {"enable_dollar_delimiter": True}},
    )

    extra = ""
    if "math/tex" in md:
        katex_css = await read_tpl("katex/katex.min.b64_fonts.css")
        katex_js = await read_tpl("katex/katex.min.js")
        mathtex_js = await read_tpl("katex/mathtex-script-type.min.js")
        extra = (
            f'<style type="text/css">{katex_css}</style>'
            f"<script defer>{katex_js}</script>"
            f"<script defer>{mathtex_js}</script>"
        )

    css = await read_tpl("markdown.css") + await read_tpl("pygments-default.css")
    html = await text_template.render_async(
        md=md, font_path=font_path, css=css, extra=extra
    )
    # with open(Path() / "temp.html", "w") as f:
    #     f.write(html)
    return await html_to_pic(
        html=html,
        viewport={"width": width, "height": 10},
    )



import asyncio
from pathlib import Path
from typing import Annotated

from nonebot import require
from nonebot.exception import MatcherException
from nonebot.matcher import Matcher
from nonebot.params import ArgStr, CommandArg
from nonebot.params import CommandStart
from nonebot.plugin import on_command
from nonebot.typing import T_State

require("nonebot_plugin_templates")

from nonebot_plugin_templates.template_types import *
from nonebot_plugin_templates.templates_render import menu_render, dict_render

from .userlinks import users
from .utils import (
    send_img,
    set_common_userinfo,
    set_userinfo,
    delete_messages,
    if_super_user,
    is_super_user,
    if_close,
    send_message,
    MessageEvent,
    Message,
    Bot,
)
from ...common.mytypes import CommonUserInfo
from ...common.prefix_data import prefixes
from ...common.prompt_data import prompts
from ...common.user_data import common_users
from ...common.web.app import start_web_ui, stop_web_ui, HOST, PORT

Help_Msg_Path = Path(__file__).parent / "HelpMsg.jpeg"
Help_Msg_Path.touch()
Super_Msg_Path = Path(__file__).parent / "SuperMsg.jpeg"
Super_Msg_Path.touch()

help = on_command("shelp", aliases={"s帮助", "sparkhelp"}, priority=1, block=False)


@help.handle()
async def help_(
        matcher: Matcher, event: MessageEvent, bot: Bot, foo: Annotated[str, CommandStart()]
):
    from ...common.load_config import (
        PRIVATE_COMMAND,
        PUBLIC_COMMAND,
        Generated_Super_Msg_Pic,
        get_super_pic,
        Generated_Help_Msg_Pic,
        get_help_pic
    )
    command_start = str(foo)
    if is_super_user(event, bot):
        if not Generated_Super_Msg_Pic:
            menu = Menu("私有bot", des="使用和管理自己独有的bot的命令,私有bot只有主人可使用,其他人无法使用",
                        funcs=Funcs(Func(f"{PRIVATE_COMMAND}bot名称+询问的问题",
                                         "与指定属于自己的bot对话\n(可使用'回复'某bot最后一个答案来连续和它对话)\n(可回复'清除历史','刷新对话'来清除bot的对话记忆)\n(可以通过发送'创造','平衡','精确'切换bing模式)") +
                                    Func(f"{PRIVATE_COMMAND}所有bot",
                                         "查询所有的可用的私有的bot,以获取bot名称和相关信息") +
                                    Func(f"{PRIVATE_COMMAND}创建bot", "创建新的私有的bot") +
                                    Func(f"{PRIVATE_COMMAND}改名bot", "更改自己的bot的名称") +
                                    Func(f"{PRIVATE_COMMAND}删除bot", "删除指定自己的bot")))
            menu += Menu("公有bot", des="使用和管理公有的bot的命令",
                         funcs=Funcs(
                             Func(f"{PUBLIC_COMMAND}bot名称+询问的问题",
                                  "与指定属于公共的bot对话\n(可使用'回复'某bot最后一个答案来连续和它对话)\n(可回复'清除历史','刷新对话'来清除bot的对话记忆)\n(可以通过发送'创造','平衡','精确'切换bing模式)") +
                             Func(f"{PUBLIC_COMMAND}所有bot", "查询所有的可用的公共的bot,以获取bot名称和相关信息") +
                             Func(f"{PUBLIC_COMMAND}创建bot", "创建新的公用的bot") +
                             Func(f"{PUBLIC_COMMAND}改名bot", "更改公用的bot的名称") +
                             Func(f"{PUBLIC_COMMAND}删除bot", "删除指定公用的bot")))
            menu += Menu("预设", des="查看和管理预设",
                         funcs=Funcs(
                             Func(f"{command_start}所有预设", "列出所有预设的名称和缩略的内容") +
                             Func(f"{command_start}查询预设", "查询指定预设的具体详尽内容") +
                             Func(f"{command_start}添加预设", "添加新的预设") +
                             Func(f"{command_start}删除预设", "删除指定预设") +
                             Func(f"{command_start}改名预设", "修改指定预设的名字")))
            menu += Menu("前缀", des="查看和管理前缀",
                         funcs=Funcs(Func(f"{command_start}所有前缀", "列出所有前缀的的名称和缩略的内容") +
                                     Func(f"{command_start}查询前缀", "查询指定前缀的具体详尽内容") +
                                     Func(f"{command_start}添加前缀", "添加新的前缀") +
                                     Func(f"{command_start}删除前缀", "删除指定前缀") +
                                     Func(f"{command_start}改名前缀", "修改指定前缀的名字")))
            menu += Menu("账户信息", des="用来实现跨平台(qq,tg等)同账户身份,同步你的数据",
                         funcs=Funcs(Func(f"{command_start}用户信息",
                                          "查询当前用户的通用用户的用户名和密钥.(5秒后撤回,但建议私聊使用)") +
                                     Func(f"{command_start}更改绑定",
                                          "将当前平台账户绑定到指定通用账户,实现跨平台数据互通")))
            menu += Menu("webui", des="管理WebUi",
                         funcs=Funcs(
                             Func(f"{command_start}开启webui", "默认开启,打开webui,并返回webui开启的端口(管理员可用)") +
                             Func(f"{command_start}关闭webui", "请在使用webui后关闭(管理员可用)")))
            pic_bytes = await menu_render(menu, 800)
            with open(Super_Msg_Path, "wb") as f:
                f.write(pic_bytes)
            get_super_pic()
        else:
            with open(Super_Msg_Path, "rb") as f:
                pic_bytes = f.read()
        await send_img(pic_bytes, matcher, bot, event)
        await matcher.finish()
    else:
        if not Generated_Help_Msg_Pic:
            menu = Menu("私有bot", des="使用和管理自己独有的bot的命令,私有bot只有主人可使用,其他人无法使用",
                        funcs=Funcs(Func(f"{PRIVATE_COMMAND}bot名称+询问的问题",
                                         "与指定属于自己的bot对话\n(可使用'回复'某bot最后一个答案来连续和它对话)\n(可回复'清除历史','刷新对话'来清除bot的对话记忆)\n(可以通过发送'创造','平衡','精确'切换bing模式)") +
                                    Func(f"{PRIVATE_COMMAND}所有bot",
                                         "查询所有的可用的私有的bot,以获取bot名称和相关信息") +
                                    Func(f"{PRIVATE_COMMAND}创建bot", "创建新的私有的bot") +
                                    Func(f"{PRIVATE_COMMAND}改名bot", "更改自己的bot的名称") +
                                    Func(f"{PRIVATE_COMMAND}删除bot", "删除指定自己的bot")))
            menu += Menu("公有bot", des="使用和管理公有的bot的命令",
                         funcs=Funcs(
                             Func(f"{PUBLIC_COMMAND}bot名称+询问的问题",
                                  "与指定属于公共的bot对话\n(可使用'回复'某bot最后一个答案来连续和它对话)\n(可回复'清除历史','刷新对话'来清除bot的对话记忆)\n(可以通过发送'创造','平衡','精确'切换bing模式)") +
                             Func(f"{PUBLIC_COMMAND}所有bot", "查询所有的可用的公共的bot,以获取bot名称和相关信息")))
            menu += Menu("预设", des="查看和管理预设",
                         funcs=Funcs(
                             Func(f"{command_start}所有预设", "列出所有预设的名称和缩略的内容") +
                             Func(f"{command_start}查询预设", "查询指定预设的具体详尽内容")))
            menu += Menu("前缀", des="查看和管理前缀",
                         funcs=Funcs(Func(f"{command_start}所有前缀", "列出所有前缀的的名称和缩略的内容") +
                                     Func(f"{command_start}查询前缀", "查询指定前缀的具体详尽内容")))
            menu += Menu("账户信息", des="用来实现跨平台(qq,tg等)同账户身份,同步你的数据",
                         funcs=Funcs(Func(f"{command_start}用户信息",
                                          "查询当前用户的通用用户的用户名和密钥.(5秒后撤回,但建议私聊使用)") +
                                     Func(f"{command_start}更改绑定",
                                          "将当前平台账户绑定到指定通用账户,实现跨平台数据互通")))
            pic_bytes = await menu_render(menu, 700)

            with open(Help_Msg_Path, "wb") as f:
                f.write(pic_bytes)
            get_help_pic()
        else:
            with open(Help_Msg_Path, "rb") as f:
                pic_bytes = f.read()
        await send_img(pic_bytes, matcher, bot, event)
        await matcher.finish()


start_web_ui_ = on_command("开启webui", priority=1, block=False)


@start_web_ui_.handle()
async def start_web_ui__(matcher: Matcher, event: MessageEvent, bot: Bot):
    await if_super_user(event, bot, matcher)
    await start_web_ui()
    msg = f"成功开启webui,地址为http://{HOST}:{PORT},请在使用完成后关闭,以免他人修改内容"
    await send_message(msg, matcher, bot, event)
    await matcher.finish()


stop_web_ui___ = on_command("关闭webui", priority=1, block=False)


@stop_web_ui___.handle()
async def stop_web_ui__(matcher: Matcher, event: MessageEvent, bot: Bot):
    await if_super_user(event, bot, matcher)
    await stop_web_ui()
    await send_message("成功关闭webui", matcher, bot, event)
    await matcher.finish()


userinfo = on_command("用户信息", priority=1, block=False)


@userinfo.handle()
async def userinfo_(matcher: Matcher, event: MessageEvent, bot: Bot):
    common_userinfo = set_common_userinfo(event, bot)
    common_user_id = common_userinfo.user_id
    common_user_key = common_users.get_key(common_userinfo)
    msg = f"通用用户名为:{common_user_id}\n用户密钥为:{common_user_key}\n该消息将在5秒后尝试被撤回,请及时保存"
    reply = await send_message(msg, matcher, bot, event)
    await asyncio.sleep(5)
    await delete_messages(bot, event, [reply])
    await matcher.finish()


user_relink = on_command("更改绑定", priority=1, block=False)


@user_relink.handle()
async def user_relink_(
        matcher: Matcher,
        state: T_State,
        event: MessageEvent,
        bot: Bot,
        args: Message = CommandArg(),
):
    reply_msgs = []
    if not args:
        msg = "请输入用户名和用户密钥,用空格分隔开\n输入'取消'或'算了'可以结束当前操作"
        reply_msgs.append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("infos", args)
    state["replys"] = reply_msgs


@user_relink.got("infos")
async def user_relink__(
        matcher: Matcher,
        event: MessageEvent,
        state: T_State,
        bot: Bot,
        args: str = ArgStr("infos"),
):
    await if_close(event, matcher, bot, state["replys"])
    reply_msgs = state["replys"]
    infos = str(args).split(" ", 1)
    if len(infos) == 2:
        common_user_id, common_user_key = infos
    else:
        await delete_messages(bot, event, reply_msgs)
        msg = "你输入的信息格式有误,请重新发送命令和信息"
        await send_message(msg, matcher, bot, event)
        await matcher.finish()
    common_userinfo = CommonUserInfo(user_id=common_user_id)
    if common_userinfo in list(common_users.user_dict.keys()):
        if common_users.get_key(common_userinfo) != common_user_key:
            await delete_messages(bot, event, reply_msgs)
            msg = "你输入的密钥错误,请重新发送指令和信息"
            await send_message(msg, matcher, bot, event)
            await matcher.finish()

        else:
            userinfo = set_userinfo(event, bot)
            origin_common_userinfo = set_common_userinfo(event, bot)
            users.link(userinfo, common_userinfo)
            common_users.if_delete_user(
                common_userinfo=origin_common_userinfo, userinfo=userinfo
            )
            await delete_messages(bot, event, reply_msgs)
            await send_message("成功绑定至指定用户", matcher, bot, event)
            await matcher.finish()

    else:
        await delete_messages(bot, event, reply_msgs)
        msg = "你输入的用户名不存在,请重新发送指令和信息"
        await send_message(msg, matcher, bot, event)
        await matcher.finish()


all_prompts = on_command("所有预设", priority=1, block=False)

Prompt_Msg_Path = Path(__file__).parent / "PromptMsg.jpeg"
Prompt_Msg_Path.touch()


@all_prompts.handle()
async def all_prompts_(bot: Bot, matcher: Matcher, event: MessageEvent):
    prompts_dict = prompts.show_list()
    if not prompts.Generated:
        pic_bytes = await dict_render(prompts_dict, title="预设列表", width=600)
        prompts.generate_pic()
        with open(Prompt_Msg_Path, "wb") as f:
            f.write(pic_bytes)
    else:
        with open(Prompt_Msg_Path, "rb") as f:
            pic_bytes = f.read()
    await send_img(
        pic_bytes, matcher, bot, event,
    )
    await matcher.finish()


show_prompt = on_command("查询预设", priority=1, block=False)


@show_prompt.handle()
async def show_prompt_(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: Message = CommandArg(),
):
    state["replys"] = []
    if not args:
        msg = "请输入预设的名称,区分大小写\n输入'取消'或'算了'可以结束当前操作"
        state["replys"].append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("prompt", args)


@show_prompt.got("prompt")
async def show_prompt__(
        matcher: Matcher,
        state: T_State,
        event: MessageEvent,
        bot: Bot,
        args: str = ArgStr("prompt"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prompt = prompts.show_prompt(str(args))
        await send_message(prompt, matcher, bot, event, plain=False)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


delete_prompt = on_command("删除预设", priority=1, block=False)


@delete_prompt.handle()
async def delete_prompt_(
        matcher: Matcher,
        state: T_State,
        event: MessageEvent,
        bot: Bot,
        args: Message = CommandArg(),
):
    await if_super_user(event, bot, matcher)
    state["replys"] = []
    if not args:
        msg = "请输入预设的名称,区分大小写\n输入'取消'或'算了'可以结束当前操作"
        state["replys"].append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("prompt", args)


@delete_prompt.got("prompt")
async def delete_prompt__(
        matcher: Matcher,
        event: MessageEvent,
        state: T_State,
        bot: Bot,
        args: str = ArgStr("prompt"),
):
    await if_close(event, matcher, bot, state["replys"])
    prompt_name = str(args).replace("\n", "")
    try:
        prompts.delete(prompt_name)
        await send_message("成功删除了该预设", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


add_prompt = on_command("添加预设", priority=1, block=False)


@add_prompt.handle()
async def add_prompt_(matcher: Matcher, event: MessageEvent, bot: Bot, state: T_State):
    await if_super_user(event, bot, matcher)
    msg = "请输入预设的要设为的名称\n输入'取消'或'算了'可以结束当前操作"
    state["replys"] = []
    state["replys"].append(await send_message(msg, matcher, bot, event))


@add_prompt.got("prompt_name")
async def add_prompt__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prompt_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    state["prompt_name"] = str(args)
    msg = "请输入预设的内容\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@add_prompt.got("prompt")
async def add_prompt___(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prompt"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prompts.add(state["prompt_name"], str(args))
        await send_message("成功添加了对应的预设", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


change_prompt_name = on_command("改名预设", priority=1, block=False)


@change_prompt_name.handle()
async def change_prompt_name_(
        matcher: Matcher, bot: Bot, event: MessageEvent, state: T_State
):
    await if_super_user(event, bot, matcher)
    state["replys"] = []
    msg = "请输入要更改的预设的名称\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@change_prompt_name.got("prompt_name")
async def change_prompt_name__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prompt_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    state["prompt_name"] = str(args)
    msg = "请输入预设要改为的名字\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@change_prompt_name.got("new_prompt_name")
async def change_prompt_name___(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("new_prompt_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prompts.rename(old_name=state["prompt_name"], new_name=str(args))
        await send_message("成功更改了对应的预设的名称", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


all_prefixes = on_command("所有前缀", priority=1, block=False)

Prefix_Msg_Path = Path(__file__).parent / "PrefixMsg.jpeg"
Prefix_Msg_Path.touch()


@all_prefixes.handle()
async def all_prefixes_(bot: Bot, matcher: Matcher, event: MessageEvent):
    prefixes_dict = prefixes.show_list()
    if not prefixes.Generated:
        pic_bytes = await dict_render(prefixes_dict, title="前缀列表", width=600)
        prefixes.generate_pic()
        with open(Prefix_Msg_Path, "wb") as f:
            f.write(pic_bytes)
    else:
        with open(Prefix_Msg_Path, "rb") as f:
            pic_bytes = f.read()
    await send_img(
        pic_bytes, matcher, bot, event,
    )
    await matcher.finish()


show_prefix = on_command("查询前缀", priority=1, block=False)


@show_prefix.handle()
async def show_prefix_(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: Message = CommandArg(),
):
    state["replys"] = []
    if not args:
        msg = "请输入前缀的名称,区分大小写\n输入'取消'或'算了'可以结束当前操作"
        state["replys"].append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("prefix", args)


@show_prefix.got("prefix")
async def show_prefix__(
        matcher: Matcher,
        state: T_State,
        event: MessageEvent,
        bot: Bot,
        args: str = ArgStr("prefix"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prefix = prefixes.show_prefix(str(args))
        await send_message(prefix, matcher, bot, event, plain=False)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


delete_prefix = on_command("删除前缀", priority=1, block=False)


@delete_prefix.handle()
async def delete_prefix_(
        matcher: Matcher,
        state: T_State,
        event: MessageEvent,
        bot: Bot,
        args: Message = CommandArg(),
):
    await if_super_user(event, bot, matcher)
    state["replys"] = []
    if not args:
        msg = "请输入前缀的名称,区分大小写\n输入'取消'或'算了'可以结束当前操作"
        state["replys"].append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("prefix", args)


@delete_prefix.got("prefix")
async def delete_prefix__(
        matcher: Matcher,
        event: MessageEvent,
        state: T_State,
        bot: Bot,
        args: str = ArgStr("prefix"),
):
    await if_close(event, matcher, bot, state["replys"])
    prefix_name = str(args).replace("\n", "")
    try:
        prefixes.delete(prefix_name)
        await send_message("成功删除了该前缀", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


add_prefix = on_command("添加前缀", priority=1, block=False)


@add_prefix.handle()
async def add_prefix_(matcher: Matcher, event: MessageEvent, bot: Bot, state: T_State):
    await if_super_user(event, bot, matcher)
    state["replys"] = []
    msg = "请输入前缀的要设为的名称\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@add_prefix.got("prefix_name")
async def add_prefix__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prefix_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    state["prefix_name"] = str(args)
    msg = "请输入前缀的内容\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@add_prefix.got("prefix")
async def add_prefix___(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prefix"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prefixes.add(state["prefix_name"], str(args))
        await send_message("成功添加了对应的前缀", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


change_prefix_name = on_command("改名前缀", priority=1, block=False)


@change_prefix_name.handle()
async def change_prefix_name_(
        matcher: Matcher, bot: Bot, event: MessageEvent, state: T_State
):
    await if_super_user(event, bot, matcher)
    state["replys"] = []
    msg = "请输入要更改的前缀的名称\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@change_prefix_name.got("prefix_name")
async def change_prefix_name__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prefix_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    state["prefix_name"] = str(args)
    msg = "请输入前缀要改为的名字\n输入'取消'或'算了'可以结束当前操作"
    state["replys"].append(await send_message(msg, matcher, bot, event))


@change_prefix_name.got("new_prefix_name")
async def change_prefix_name___(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("new_prefix_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    try:
        prefixes.rename(old_name=state["prefix_name"], new_name=str(args))
        await send_message("成功更改了对应的前缀的名称", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()



import json
from pathlib import Path
from typing import Dict, List, Literal, Optional, Set

from pydantic import BaseModel

PLATFORM = Literal["OneBot V11", "Telegram", "Kaiheila", "Discord"]
SOURCE = Literal[
    "poe chatgpt",
    "poe claude",
    "poe chatgpt4",
    "poe claude-2-100k",
    "chatgpt web",
    "slack claude",
    "claude ai",
    "spark desk",
    "bing",
    "sydneybing",
    "bard",
    "通义千问",
]


class BotData(BaseModel):
    """Bot的数据类型"""

    """poe"""
    handle: Optional[str]
    model: Optional[str]
    """chatgpt web"""
    conversation_id: Optional[str]
    parent_id: Optional[str]
    """通义千问"""
    sessionId: Optional[str]
    parentMsgId: Optional[str]
    userId: Optional[str]
    """slack Claude"""
    msg_ts: Optional[str]
    thread_ts: Optional[str]
    """Claude Ai"""
    conversation_uuid: Optional[str]
    """spark desk"""
    chatid: Optional[str]
    """通用"""
    prefix_nickname: Optional[str]
    prefix: Optional[str]
    nickname: Optional[str]
    owner: Optional[str]
    is_waiting: Optional[bool]
    last_suggests: Optional[List[str]] = []
    source: SOURCE
    prompt_nickname: Optional[str]
    prompt: Optional[str]

    def __hash__(self):
        return hash((self.nickname, self.owner))

    def save(self) -> dict:
        """
        将对象转换为字典

        :return: 转换后的字典
        :rtype: dict
        """
        return self.dict()

    @classmethod
    def load(cls, data: dict) -> "BotInfo":
        """
        根据字典创建对象

        :param data: 字典数据
        :type data: dict
        :return: 创建的对象
        :rtype: BotInfo
        """
        return cls(**data)


class UserInfo(BaseModel):
    """用户的平台信息类型

    :param platform: 用户所在平台
    :type platform: PLATFORM
    :param username: 用户名(必须在本平台唯一)
    :type username: str
    """

    platform: PLATFORM
    username: str

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.platform, self.username))

    def save(self) -> str:
        return f"{self.platform}-{self.username}"

    @classmethod
    def load(cls, userinfo: str) -> "UserInfo":
        platform, username = userinfo.split("-")
        return cls(platform=platform, username=str(username))


class CommonUserInfo(BaseModel):
    """用户通用身份

    :param user_id: 用户的唯一特征id
    :type user_id: str
    """

    user_id: str

    def __hash__(self):
        return hash(self.user_id)

    def save(self) -> str:
        return self.user_id

    @classmethod
    def load(cls, user_id: str) -> "CommonUserInfo":
        return cls(user_id=user_id)


class BotInfo(BaseModel):
    """Bot的信息

    :param nickname: Bot的别名
    :type nickname: str
    """

    nickname: str
    owner: CommonUserInfo

    def __hash__(self):
        return hash(self.nickname)

    def save(self) -> str:
        return f"{self.nickname}-{self.owner.user_id}"

    @classmethod
    def load(cls, arg: str) -> "BotInfo":
        args = arg.split("-", 1)
        return cls(nickname=args[0], owner=CommonUserInfo(user_id=args[1]))


class CommonUserData(BaseModel):
    """用户数据

    :param bots: 用户的所有bots
    :type bots: Dict[BotInfo, BotData]
    :param key: 用户的验证密钥
    :type key: str
    """

    bots: Dict[BotInfo, BotData]
    user_id: str
    key: str

    def __hash__(self):
        return hash(())

    def save(self) -> Dict:
        data = {
            "bots": {},
            "user_id": self.user_id,
            "key": self.key,
        }
        for botinfo, botdata in self.bots.items():
            data["bots"][botinfo.save()] = botdata.save()
        path: Path = Path() / f"data/spark_gpt/common/common_users/{self.user_id}.json"
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch()
        with open(path, "w") as f:
            json.dump(data, f)
        return data

    @classmethod
    def load(cls, data: Dict) -> "CommonUserData":
        newdata = {"bots": {}, "user_id": data["user_id"], "key": data["key"]}
        for botinfo, botdata in data["bots"].items():
            newdata["bots"][BotInfo.load(botinfo)] = BotData.load(botdata)
        return cls(**newdata)


class UsersInfo(BaseModel):
    users: Set[UserInfo]

    def __hash__(self):
        # 将所有用户信息的哈希值相加得到 Usersinfo 的哈希值
        return sum(hash(user) for user in self.users)

    def save(self) -> list:
        data = [user.save() for user in self.users]
        return data

    @classmethod
    def load(cls, usersinfo: List[str]) -> "UsersInfo":
        return UsersInfo(users=[UserInfo.load(userinfo) for userinfo in usersinfo])
